
/* Chrysalide - Outil d'analyse de fichiers binaires
 * logs.h - prototypes pour la diffusion de messages d'alerte ou informatifs
 *
 * Copyright (C) 2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _CORE_LOGS_H
#define _CORE_LOGS_H


#include <i18n.h>



/* Type de messages disponibles */
typedef enum _LogMessageType
{
    LMT_INFO,                               /* Information sur l'exécution */
    LMT_PROCESS,                            /* Début de tâche quelconque   */
    LMT_WARNING,                            /* Avertissment à remonter     */
    LMT_ERROR,                              /* Erreur de traitement        */
    LMT_BAD_BINARY,                         /* Binaire malformé            */

    LMT_COUNT

} LogMessageType;


/* Fournit la verbosité des messages système. */
LogMessageType get_log_verbosity(void);

/* Définit la verbosité des messages système. */
void set_log_verbosity(LogMessageType);

/* Affiche un message dans le journal des messages système. */
void log_simple_message(LogMessageType, const char *);

/* Affiche un message dans le journal des messages système. */
void log_variadic_message(LogMessageType, const char *, ...);



#endif  /* _CORE_LOGS_H */
